/* $Id: mac.h 594 2008-07-21 09:50:36Z gloomy $ */
#ifndef _SYS_MAC_H_
#define _SYS_MAC_H_

#include <wchar.h>
#include <sys/fcntl.h>
#include <stdint.h>

//#include <parsec/parsec.h> /* Remove after full system rebuild  */
//#include <parsec/parsec_mac.h> /* Remove after full system rebuild */

#ifndef __MAC_TYPES
#define __MAC_TYPES
typedef uint8_t		mac_lev_t; /* must be removed from db ? */
typedef uint64_t	mac_cat_t;
typedef uint32_t	mac_type_t; /* label, context and so on... */
#endif

#define MAC_TYPE_VAL(v)		((v)>>16)
#define MAC_TYPE(v)		((v)&0xf0000)
#define MAC_TYPE_SUBJECT	0x00000 /* null flags */
#define MAC_TYPE_EQU_W		(MAC_TYPE_OBJECT | MAC_ATTR_IGNOREW)
#define MAC_TYPE_EQU		(MAC_TYPE_OBJECT | MAC_ATTR_IGNOREW | MAC_ATTR_IGNORER | MAC_ATTR_IGNOREX)
#define MAC_TYPE_LOW		(MAC_TYPE_OBJECT | MAC_ATTR_IGNORER | MAC_ATTR_IGNOREX)
#define MAC_TYPE_OBJECT		0x10000

struct __mac;
typedef struct __mac  	*mac_t; /* for subjects AND objects */

#ifdef __cplusplus
extern "C" {
#endif
/* parsec mandate functions */
/* example 

	mac = mac_init(MAC_TYPE_SUBJECT);
	mac_set_lev(mac, 1);
	mac_set_cat(mac, 2);
	mac_set_pid(10);
	mac_free(mac);
	
*/
	extern mac_t	mac_init(mac_type_t);
	extern mac_t 	mac_dup(mac_t);
	extern int	mac_copy(mac_t dst, mac_t src); /* may be type differ */
	extern int 	mac_cmp(mac_t src, mac_t dst);
	extern int	mac_valid(mac_t);

	extern mac_t	mac_copy_int(const void *buf); /* from raw to int */
	extern ssize_t 	mac_copy_ext(void *buf, mac_t mac, size_t size); /* from int to raw */

#define MAC_FMT_NUM		0
#define MAC_FMT_TXT		1
#define MAC_FMT_LEV		2
#define MAC_FMT_CAT		4
#define MAC_FMT_TYPE		8

	extern char *	mac_to_text(mac_t, ssize_t *size, int flags); /* to text */
	wchar_t *	mac_to_wtext(mac_t mac, ssize_t *size, int flags); /* to wtext */
	
	extern int 	mac_from_text(mac_t mac, const char *text); /* from text */
	extern ssize_t	mac_size(mac_t mac); /* get external size */

	extern int	mac_set_lev(mac_t mac, mac_lev_t);
	extern int	mac_set_cat(mac_t mac, mac_cat_t);
	extern int	mac_set_type(mac_t mac, mac_type_t);

	extern int	mac_set_file(const char *filename, mac_t mac); /* a la acl */
	extern int	mac_set_fd(int fd, mac_t mac);
	extern mac_t	mac_get_file(const char *filename);
	extern int	mac_empty(mac_t mac);
	extern mac_t	mac_get_fd(int fd);

	extern int	mac_set_pid(pid_t pid, mac_t mac);
	extern mac_t	mac_get_pid(pid_t pid);
	
	static inline int mac_set_proc(mac_t mac) { return mac_set_pid(0, mac); }
	static inline mac_t mac_get_proc(void) { return mac_get_pid(0); }
	
	extern mac_lev_t mac_get_lev(mac_t mac);
	extern mac_cat_t mac_get_cat(mac_t mac);
	extern mac_type_t mac_get_type(mac_t mac);
	extern int	mac_permit(mac_t m1, mac_t m2, int op); /* op: X_OK, R_OK, W_OK */
	extern int	mac_file_permit(mac_t m1, mac_t m2, int op); /* op: X_OK, R_OK, W_OK */
	extern void	mac_free(mac_t); 

#ifdef __cplusplus
}
#endif
#endif				/* _SYS_MAC_H_ */
