/* $Id: mac_db.h 484 2007-11-20 16:17:06Z pkun $ */

#ifndef _PARSEC_MAC_DB_H_
#define _PARSEC_MAC_DB_H_

#include <pwd.h>
#include <sys/stat.h>

#include <parsec/parsec_mac.h>

#define MAC_MAX_LEV          MAC_MAX_LEVEL
#define MAC_MAX_CAT          MAC_MAX_CATEGORY

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************
 *           WORKING WITH CONFIG FILES             *
 ***************************************************/

//#define MAKE_SECRET_CATALOGS
//#define WORDS_LEVELS_CATEGORIES 
#define SECURITY "security"

#ifndef __MAC_TYPES
#define __MAC_TYPES
	typedef uint8_t		mac_lev_t; /* must be removed from db ? */
	typedef uint64_t	mac_cat_t;
	typedef uint32_t	mac_type_t; /* label, context and so on... */
#endif

	struct mac_label {
		mac_lev_t lev;
		mac_cat_t cat;
	};

	struct mac_user {
		char *name;
		struct mac_label min;
		struct mac_label max;
	};

	struct mac_lev {
		char *name;
		mac_lev_t value;
	};

	struct mac_cat {
		char *name;
		mac_cat_t value;
	};
	
/* mac */
	struct __mac_han_t;
	typedef struct __mac_han_t *mac_han_t;

	extern struct mac_user *parse_mac(const char *buff, 
		struct mac_user *mu);
	extern char *compose_mac(const struct mac_user *mu);

	// thread-safe base API
	extern mac_han_t openmacdb_r(void);
	extern int closemacdb_r(mac_han_t han);

	extern void freemacent_r(struct mac_user *res);
	extern int setmacent_r(mac_han_t han);
	extern int endmacent_r(mac_han_t han);
	extern struct mac_user *getmacent_r(mac_han_t han);
	extern struct mac_user *getmacnam_r(mac_han_t han, 
		const char *name);
	extern struct mac_user *getmacuid_r(mac_han_t han, uid_t uid);
	extern int rmmacnam_r(mac_han_t han, const char *name);
	extern int putmacent_r(mac_han_t han, const struct mac_user *mu);

	extern struct mac_user *fgetmacent_r(FILE *fd);
	extern int fputmacent_r(FILE *fd, const struct mac_user *mu);

	extern int getlchuid(uid_t uid, struct mac_label* mac);
	extern int getlchnam(const char* name,struct mac_label* mac);
	extern int setlchuid(uid_t uid, const struct mac_label* mac);
	extern int setlchnam(const char* name,const struct mac_label* mac);


	// common old API
	extern int setmacent(void);
	extern void endmacent(void);
	extern struct mac_user *getmacent(void);
	extern struct mac_user *getmacnam(const char *name);
	extern struct mac_user *getmacuid(uid_t uid);
	extern int putmacent(const struct mac_user *mu);
	extern struct mac_user *fgetmacent(FILE * f);
	extern void fputmacent(FILE * f, const struct mac_user *mu);
	extern int rmmacnam(const char *name);

/* levels */
	struct __lev_han_t;
	typedef struct __lev_han_t *lev_han_t;

	extern struct mac_lev *parse_lev(const char *buff, 
		struct mac_lev *lev);
	extern char *compose_lev(const struct mac_lev *lev);

	// thread-safe base API
	extern lev_han_t openlevdb_r(void);
	extern int closelevdb_r(lev_han_t han);

	extern void freelevent_r(struct mac_lev *lev);
	extern int setlevent_r(lev_han_t han);
	extern int endlevent_r(lev_han_t han);
	extern struct mac_lev *getlevent_r(lev_han_t han);
	extern struct mac_lev *getlevnam_r(lev_han_t han, const char *name);
	extern struct mac_lev *getlevval_r(lev_han_t han, mac_lev_t value);
	extern int putlevent_r(lev_han_t han, const struct mac_lev *ml);
	extern int rmlevnam_r(lev_han_t han, const char *name);
	extern int rmlevval_r(lev_han_t han, mac_lev_t value);
	extern int levexstval(lev_han_t ha, mac_lev_t value);

	extern struct mac_lev *fgetlevent_r(FILE * f);
	extern int fputlevent_r(FILE * f, const struct mac_lev *ml);

	// common old API
	extern int setlevent(void);
	extern void endlevent(void);
	extern struct mac_lev *getlevent(void);
	extern struct mac_lev *getlevnam(const char *name);
	extern struct mac_lev *getlevval(mac_lev_t value);
	extern int putlevent(const struct mac_lev *ml);
	extern struct mac_lev *fgetlevent(FILE * f);
	extern void fputlevent(FILE * f, const struct mac_lev *ml);
	extern int rmlevnam(const char *name);
	extern int rmlevval(mac_lev_t value);

/* categories */
	struct __cat_han_t;
	typedef struct __cat_han_t *cat_han_t;

	extern struct mac_cat *parse_cat(const char *buff, 
		struct mac_cat *cat);
	extern char *compose_cat(const struct mac_cat *cat);

	// thread-safe base API
	extern cat_han_t opencatdb_r(void);
	extern int closecatdb_r(cat_han_t han);

	extern void freecatent_r(struct mac_cat *cat);
	extern int setcatent_r(cat_han_t han);
	extern int endcatent_r(cat_han_t han);
	extern struct mac_cat *getcatent_r(cat_han_t han);
	extern struct mac_cat *getcatnam_r(cat_han_t han, const char *name);
	extern struct mac_cat *getcatval_r(cat_han_t han, mac_cat_t value);
	extern int putcatent_r(cat_han_t han, const struct mac_cat *mc);
	extern int rmcatnam_r(cat_han_t han, const char *name);
	extern int rmcatval_r(cat_han_t han, mac_cat_t value);
	extern int is_cats_valid(cat_han_t han, mac_cat_t cats);

	extern struct mac_cat *fgetcatent_r(FILE * fd);
	extern int fputcatent_r(FILE * fd, const struct mac_cat *mc);

	// common old API
	extern int setcatent(void);
	extern void endcatent(void);
	extern struct mac_cat *getcatent(void);
	extern struct mac_cat *getcatnam(const char *name);
	extern struct mac_cat *getcatval(mac_cat_t value);
	extern int putcatent(const struct mac_cat *mc);
	extern struct mac_cat *fgetcatent(FILE * f);
	extern void fputcatent(FILE * f, const struct mac_cat *mc);
	extern int rmcatnam(const char *name);
	extern int rmcatval(mac_cat_t value);

#ifdef __cplusplus
}
#endif
#endif	/* _PARSEC_MAC_DB_H_ */
