/* $Id: parsec_mac.h 194 2007-04-28 13:28:52Z pkun $ */

#ifndef _PARSEC_MAC_H_
#define _PARSEC_MAC_H_

#include <parsec/mac.h>
#include <parsec/parsec.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef uint8_t	mic_t;

typedef uint8_t	parsec_lev_t;
typedef uint64_t parsec_cat_t;

typedef struct _parsec_mac_t {
	parsec_lev_t lev;
	parsec_cat_t cat;
} parsec_mac_t;			// for subjects

typedef struct _parsec_mac_label_t {
	uint32_t type;
	parsec_mac_t mac;
} parsec_mac_label_t;		// for objects



#define MAC_TYPE_ATTR(v)	(v&0xff)

#ifndef MAC_ATTR_IGNORER_CAT
#define MAC_ATTR_IGNORER_CAT 0x01
#endif

#ifndef MAC_ATTR_IGNOREW_CAT
#define MAC_ATTR_IGNOREW_CAT 0x02
#endif

#ifndef MAC_ATTR_IGNOREX_CAT
#define MAC_ATTR_IGNOREX_CAT 0x04
#endif

#ifndef MAC_ATTR_IGNORER_LVL
#define MAC_ATTR_IGNORER_LVL 0x08
#endif

#ifndef MAC_ATTR_IGNOREW_LVL
#define MAC_ATTR_IGNOREW_LVL 0x10
#endif

#ifndef MAC_ATTR_IGNOREX_LVL
#define MAC_ATTR_IGNOREX_LVL 0x20
#endif

#define MAC_EMPTY(m) (((m)->lev == 0) && ((m)->cat == 0))
#define MAC_LABEL_EMPTY(m) (MAC_EMPTY(&m->mac) && (!(m)->type))

#define MAC_ATTR_IGNORER  ((MAC_ATTR_IGNORER_CAT) | (MAC_ATTR_IGNORER_LVL))
#define MAC_ATTR_IGNOREW  ((MAC_ATTR_IGNOREW_CAT) | (MAC_ATTR_IGNOREW_LVL))
#define MAC_ATTR_IGNOREX  ((MAC_ATTR_IGNOREX_CAT) | (MAC_ATTR_IGNOREX_LVL))

#define MAC_ATTR_IGNORE ((MAC_ATTR_IGNORER) | (MAC_ATTR_IGNOREW) | (MAC_ATTR_IGNOREX))

#define MAC_ATTR_CHECK(mac, a) ((mac).attr & MAC_ATTR_##a)

#define MAC_MAX_LEVEL 255
#define MAC_MAX_CATEGORY ((uint64_t)(-1))


/* parsec mandate functions */
	extern int parsec_chmac(const char *filename, const parsec_mac_label_t *mac);
	extern int parsec_fchmac(int fd, const parsec_mac_label_t *mac);
	extern int parsec_statmac(const char *filename, parsec_mac_label_t *mac);
	extern int parsec_fstatmac(int fd, parsec_mac_label_t *mac);
	extern int parsec_setmac(pid_t pid, const parsec_mac_t *mac);
	extern int parsec_getmac(pid_t pid, parsec_mac_t *mac);


	extern char* parsec_mac_t_to_text(const parsec_mac_t *mac, mac_type_t type, int flags);
	extern int parsec_mac_t_from_text(parsec_mac_t *mac,const char* mac_txt);
	extern char* parsec_mac_label_t_to_text(const parsec_mac_label_t *mac, int flags);
	extern int parsec_mac_label_t_from_text(parsec_mac_label_t *mac,const char* mac_txt);

	extern mac_t parsec_mac_t_to_mac_t(const parsec_mac_t* pmact,mac_type_t type);
	extern void mac_t_to_parsec_mac_t(const mac_t mact, parsec_mac_t* pmact);
	extern mac_t parsec_mac_label_t_to_mac_t(const parsec_mac_label_t* pmaclt);
	extern void mac_t_to_parsec_mac_label_t(const mac_t mact, parsec_mac_label_t* pmact);

#ifdef __cplusplus
}
#endif
#endif				/* _PARSEC_MAC_H_ */
